@@ -18,9 +18,9 @@ _CMP_MAP = {
 def _binop(opname, cls=instructions.Instruction):
     def wrap(fn):
         @functools.wraps(fn)
-        def wrapped(self, lhs, rhs, name=''):
+        def wrapped(self, lhs, rhs, name='', flags=()):
             assert lhs.type == rhs.type, "Operands must be the same type"
-            instr = cls(self.block, lhs.type, opname, (lhs, rhs), name)
+            instr = cls(self.block, lhs.type, opname, (lhs, rhs), name, flags)
             self._insert(instr)
             return instr
 
@@ -125,7 +125,7 @@ class IRBuilder(object):
     def _branch_helper(self, bbenter, bbexit):
         self.position_at_end(bbenter)
         yield bbexit
-        if bbenter.terminator is None:
+        if self.basic_block.terminator is None:
             self.branch(bbexit)
 
     @contextlib.contextmanager
@@ -182,9 +182,6 @@ class IRBuilder(object):
 
         self.position_at_end(bbend)
 
-    def constant(self, typ, val):
-        return values.Constant(typ, val)
-
     def _insert(self, instr):
         self._block.instructions.insert(self._anchor, instr)
         self._anchor += 1
@@ -460,7 +457,7 @@ class IRBuilder(object):
 
     def gep(self, ptr, indices, inbounds=False, name=''):
         instr = instructions.GEPInstr(self.block, ptr, indices,
-                                inbounds=inbounds, name=name)
+                                      inbounds=inbounds, name=name)
         self._insert(instr)
         return instr
 
@@ -507,3 +504,8 @@ class IRBuilder(object):
                                     failordering, name=name)
         self._insert(inst)
         return inst
+
+    def assume(self, cond):
+        fn = self.module.declare_intrinsic("llvm.assume")
+        return self.call(fn, [cond])
+
